#include "SnGwParser.h"
#include "SnMJSONParser.h"
#include "IoTGWHandler.h"
//#include "inc/SensorNetwork_Manager_API.h"
//#include "inc/AdvWebClientAPI.h"



int ParseReceivedData(void* data, int datalen, int * cmdID, char *sessionId, int nLenSessionId )
{
	/*{"susiCommData":{"commCmd":251,"catalogID":4,"requestID":10}}*/

	cJSON* root = NULL;
	cJSON* body = NULL;
	cJSON* target = NULL;
	int ver = 1;
	if(!data) return 0;
	if(datalen<=0) return 0;
	root = cJSON_Parse((char *)data);
	if(!root) { 
		return 0;
	}

	body = cJSON_GetObjectItem(root, AGENTINFO_BODY_STRUCT);

	
	if(!body)
	{
		// Try WISE-PaaS/2.0
		body = cJSON_GetObjectItem( root, WISE_2_0_CONTENT_BODY_STRUCT);
		if( !body )
		{	
			cJSON_Delete(root);
			return 0;
		}
		else
			ver = 2;
	}

	if( ver == 1)
		target = cJSON_GetObjectItem(body, AGENTINFO_CMDTYPE);
	else
		target = cJSON_GetObjectItem(root, AGENTINFO_CMDTYPE);


	if(target)
		*cmdID = target->valueint;


	target = cJSON_GetObjectItem( body, AGENTINFO_SESSION_ID ); 

	if(target)
		snprintf(sessionId,nLenSessionId,"%s",target->valuestring );

	PRINTF("cmd=%d sessionId=%s\n", *cmdID, sessionId);
	cJSON_Delete(root);
	return 1;
}

//   topic: /cagent/admin/%s/agentcallbackreq
int GetSenHubUIDfromTopic(const char *topic, char *uid , const int size )
{
	int rc = 0;
	char *start = NULL;
	char *end = NULL;
	char *sp = NULL;
	int len = 0;

	if( topic == NULL || strlen(topic) == 0 || uid == NULL ) return rc;

	sp = strstr(topic,"admin/");
	if( sp )
		start = sp + 6; // after /

	sp = strstr(topic,"/agentcallbackreq");
	if(sp)
	end = sp;

	if( start == NULL || end == NULL ) return rc;

	len = end - start;

	if( size < len ) return rc;

	memcpy( uid, start, len );

	rc = 1;

	return rc;
}

void RePack_GetResultToWISECloud( const char *name, const char *data , char *szResult, int bufsize )
{
	int len = 0;
	char tmpbuf[MAX_FUNSET_DATA_SIZE]={0};
	//char szResult[MAX_FUNSET_DATA_SIZE]={0};

	unsigned int nStatusCode = 200;

	cJSON* root = NULL;
	cJSON  *pSubItem = NULL; 
	char* buff;

	if(szResult == NULL)
		return;

	if( data == NULL ) { 
		sprintf(szResult, REPLY_SENSOR_400ERROR, name );
		return;
	}

	root = cJSON_Parse((char *)data);
	if(!root) { 
		sprintf(szResult, REPLY_SENSOR_400ERROR, name );
		return;
	}

	pSubItem = cJSON_GetObjectItem( root, SN_STATUS_CODE ); 

	if(!pSubItem)  { 
		sprintf(szResult, REPLY_SENSOR_400ERROR, name );
		return;
	}

	nStatusCode = pSubItem->valueint;

	// In:    { "StatusCode":200, "Result":{"n":"door","v":26,"min":0, "max":100} }
	// Out: {"n":"SenData/door temp","v":26, "StatusCode": 200}
	
	// tmpbuf => "v": 26,   "sv":"123", "bv":1
	pSubItem = cJSON_GetObjectItem( root, SN_RESUTL ); 
	
	if(!pSubItem)  { 
		sprintf(szResult, REPLY_SENSOR_400ERROR, name );
		return;
	}
	buff = cJSON_Print(pSubItem);
	GetSenMLValue(buff, tmpbuf, sizeof(tmpbuf));

	if(strlen(tmpbuf) == 0 ) {
		snprintf(tmpbuf,sizeof(tmpbuf),"\"sv\":%s",buff);
	}
	free(buff);
                                                        /* { "n":"%s", %s, "StatusCode":  %d} */
	snprintf(szResult, bufsize, REPLY_SENSOR_FORMAT, name, tmpbuf, nStatusCode );
	return;
}


int RemoveJSONTag(const char *src, char *buf , int bufsize)
{
        int len = 0;
        char *sp = strchr(src,'{' );
        char *end = strrchr(src,'}' );
        if( sp && end ) {
                len = end - sp -1;
				if( len <= bufsize )                
					memcpy( buf, sp+1, len );
				else
					len = -1;
        }
        return len;
}



void RePack_GetWResultToWISECloud( const int code, const char *name, const char *data  /* In:    {"bv":1} */, 
																			 char *szResult /* Out: {"n":"SenData/door temp","v":26, "StatusCode": 200} */ , int bufsize )
{
	int len = 0;
	char tmpbuf[MAX_FUNSET_DATA_SIZE]={0};

	unsigned int nStatusCode = 200;

	if( code < 0 )
		nStatusCode = code * -1;


	if(szResult == NULL)
		return;

	if( data == NULL ) { 
		sprintf(szResult, REPLY_SENSOR_400ERROR, name );
		return;
	}

	// remove { }
	RemoveJSONTag(data, tmpbuf, MAX_FUNSET_DATA_SIZE);

                                                        /* { "n":"%s", %s, "StatusCode":  %d} */
	snprintf(szResult, bufsize, REPLY_SENSOR_FORMAT, name, tmpbuf, nStatusCode );
	return;
}



int ProcGetSenHubValue(const char *uid, const char *sessionId, const char *data, char *buffer, const int bufsize)
{
	int len = 0;
	int i =0;
	int ver = 1;	
	char URI[MAX_PATH]={0};
	char handlename[MAX_HANDL_NAME]={0};
	char tmpbuf[MAX_FUNSET_DATA_SIZE]={0};
	char sendatas[MAX_BUFFER_SIZE]={0};
	int nStatusCode = 200;


	cJSON* root = NULL;
	cJSON* body = NULL;
	cJSON* target = NULL;
	cJSON* pElement = NULL;
	cJSON* pSubItem = NULL;
	cJSON* jsName = NULL;
	int nCount = 0;

	if(!data) return len;

	root = cJSON_Parse((char *)data);
	if(!root)
		goto exit_get;

	body = cJSON_GetObjectItem(root, AGENTINFO_BODY_STRUCT);

	if(!body) 
	{
		body = cJSON_GetObjectItem(root, AGENTINFO_BODY_STRUCT);

	
		if(!body)
		{
			// Try WISE-PaaS/2.0
			body = cJSON_GetObjectItem( root, WISE_2_0_CONTENT_BODY_STRUCT);
			if( !body ) 
				goto exit_get;
			ver = 2;
		}				
	}

	if( ver == 2)
		pSubItem = cJSON_GetObjectItem( root, SEN_HANDLER_NAME ); // handlerName
	else
		pSubItem = cJSON_GetObjectItem( body, SEN_HANDLER_NAME ); 

	if(!pSubItem) goto exit_get;

	snprintf(handlename,sizeof(handlename),"%s",pSubItem->valuestring );

	target = cJSON_GetObjectItem(body, SEN_IDLIST);

	if(!target)	goto exit_get;


	pElement = cJSON_GetObjectItem( target, SENML_E_FLAG ); // "e"
	nCount = cJSON_GetArraySize(pElement);

	for( i = 0; i<nCount ; i ++ ) {
		pSubItem = cJSON_GetArrayItem(pElement, i);
		if( pSubItem == NULL ) continue;

		jsName = cJSON_GetObjectItem( pSubItem, SENML_N_FLAG );
		
		if( jsName == NULL ) continue;

		memset(tmpbuf, 0, sizeof(tmpbuf) );

		// <DevMgt>	
		if( strstr( jsName->valuestring, "PLC/SenData/version") )
			sprintf(tmpbuf,"{\"sv\":\"2.1.2\"}");
		else if ( strstr( jsName->valuestring, "PLC/SenData/temperature") )
			sprintf(tmpbuf,"{\"v\":41}");
		else if ( strstr( jsName->valuestring, "PLC/SenData/Hue") )
			sprintf(tmpbuf,"{\"v\":80}");
		else if ( strstr( jsName->valuestring, "PLC/SenData/reset") )
			sprintf(tmpbuf,"{\"bv\":false}");


		if(strlen(tmpbuf))
			RePack_GetWResultToWISECloud( nStatusCode, jsName->valuestring, tmpbuf, sendatas, sizeof(sendatas) );

		if( i != nCount -1 ) // append ","
			strcat( sendatas, "," );
	}

	/*{"sessionID":"%s", "sensorInfoList":{"e":[%s]} }*/
	snprintf( buffer, bufsize, REPLY_FORMAT, sessionId, sendatas );

exit_get:

	if( root ) cJSON_Delete(root);


	return strlen(buffer);
}




int RePack_SetResultToWISECloud( const char *name, const char *data, int *code, char *szResult, int iLenResult)
{
	int len = 0;
	char tmpbuf[MAX_FUNSET_DATA_SIZE]={0};

	unsigned int nStatusCode = 202;

	cJSON* root = NULL;
	cJSON  *pSubItem = NULL; 
	char* buff;

	if(szResult == NULL)
	{
		nStatusCode = 400;
		*code = nStatusCode;
		return nStatusCode;
	}

	if( data == NULL ) { 
		nStatusCode = 400;
		*code = nStatusCode;
		sprintf(szResult, REPLY_SENSOR_400ERROR, name );
		return nStatusCode;
	}

	root = cJSON_Parse((char *)data);
	if(!root) { 
		nStatusCode = 400;
		*code = nStatusCode;
		sprintf(szResult, REPLY_SENSOR_400ERROR, name );
		return false;
	}

	// In:    { "StatusCode":200, "Result":{"n":"door","v":26,"min":0, "max":100} }
	// Out: {"n":"SenData/door temp","v":26, "StatusCode": 200}
	
	// tmpbuf => "v": 26,   "sv":"123", "bv":1
	GetSenMLValue(data, tmpbuf, sizeof(tmpbuf));

	pSubItem = cJSON_GetObjectItem( root, SN_STATUS_CODE ); 

	if(!pSubItem) {
		nStatusCode = 400;
		*code = nStatusCode;
		sprintf(szResult, REPLY_SENSOR_400ERROR, name );
		return nStatusCode;
	}

	nStatusCode = pSubItem->valueint;
	*code = pSubItem->valueint;

	// In:    { "StatusCode":200, "Result":{"n":"door","v":26,"min":0, "max":100} }
	// Out: {"n":"SenData/door temp","v":26, "StatusCode": 200}
	
	// tmpbuf => "v": 26,   "sv":"123", "bv":1
	pSubItem = cJSON_GetObjectItem( root, SN_RESUTL ); 
	
	if(!pSubItem)  { 
		sprintf(szResult, REPLY_SENSOR_400ERROR, name );
		return nStatusCode;
	}
	buff = cJSON_Print(pSubItem);
	GetSenMLValue(buff, tmpbuf, sizeof(tmpbuf));
	if(strlen(tmpbuf) == 0 ) {
		snprintf(tmpbuf,sizeof(tmpbuf),"\"sv\":%s",buff);
	}
	free(buff);

	                                                      /* { "n":"%s", %s, "StatusCode":  %d} */
	snprintf(szResult, iLenResult, REPLY_SENSOR_FORMAT, name, tmpbuf, nStatusCode );

	return nStatusCode;
}











/* =================== Interface Get Set ================================= */

int ProcGetInterfaceValue(const char *sessionId, const char *data, char *buffer, const int bufsize)
{
	int len = 0;
	int i =0;
	char handlename[MAX_HANDL_NAME]={0};
	char tmpbuf[MAX_FUNSET_DATA_SIZE]={0};
	char sendatas[MAX_BUFFER_SIZE]={0};
	int nStatusCode = 200;
	int nCount = 0;
	cJSON* root = NULL;
	cJSON* body = NULL;
	cJSON* target = NULL;
	cJSON* pElement = NULL;
	cJSON* pSubItem = NULL;
	cJSON* jsName = NULL;


	if(!data) return len;



	root = cJSON_Parse((char *)data);
	if(!root) {
		return 0;
	}

	body = cJSON_GetObjectItem(root, AGENTINFO_BODY_STRUCT);
	if(!body)
	{
		cJSON_Delete(root);
		return 0;
	}
	pSubItem = cJSON_GetObjectItem( body, SEN_HANDLER_NAME ); 

	if(!pSubItem)
	{
		cJSON_Delete(root);
		return 0;
	}

	snprintf(handlename,sizeof(handlename),"%s",pSubItem->valuestring ); // IoTGW

	target = cJSON_GetObjectItem(body, SEN_IDLIST);
	if(!target)
	{
		cJSON_Delete(root);
		return 0;
	}

	pElement = cJSON_GetObjectItem( target, SENML_E_FLAG ); // "e"
	nCount = cJSON_GetArraySize(pElement);

	for( i = 0; i<nCount ; i ++ ) {
		pSubItem = cJSON_GetArrayItem(pElement, i);
		if( pSubItem == NULL ) continue;

		jsName = cJSON_GetObjectItem( pSubItem, SENML_N_FLAG );
		
		if( jsName == NULL ) continue;

		
		// <DevMgt>	
		if( strstr( jsName->valuestring, "DevMgt/DevMgt/Name") )
			sprintf(tmpbuf,"{\"sv\":\"PLC_Mgt\"}");
		else if ( strstr( jsName->valuestring, "DevMgt/Info/version") )
			sprintf(tmpbuf,"{\"sv\":\"1.0.1\"}");
		else if ( strstr( jsName->valuestring, "DevMgt/DevMgt/reset") )
			sprintf(tmpbuf,"{\"bv\":false}");

		RePack_GetWResultToWISECloud( nStatusCode, jsName->valuestring, tmpbuf, sendatas, sizeof(sendatas) );

	
		if( i != nCount -1 ) // append ","
			strcat( sendatas, "," );
	}

	cJSON_Delete(root);

	/*{"sessionID":"%s", "sensorInfoList":{"e":[%s]} }*/
	snprintf( buffer, bufsize, REPLY_FORMAT, sessionId, sendatas );

	return strlen(buffer);
}









